#include "BC.cuh"
#include <string>
#include <iostream>
#include <ctime>
#include <cfloat>
#include <cmath>
#include <stdio.h>
using namespace std;

int * nodes;
Edge * edges;
int N;
int M;

void buildGraph(string fpath) {
	// read as adjacent list
	FILE * f = fopen(fpath.c_str(), "r");
	fscanf(f, "%d %d", &N, &M);
	int * adjNodes = (int *)malloc(N * sizeof(int));
	AdjEdge * adjEdges = (AdjEdge *)malloc(2 * M * sizeof(AdjEdge));
	cout << "nodes: " << N << endl << "edges: " << M << endl;
	memset(adjNodes, -1, N * sizeof(int));
	memset(adjEdges, 0, M * sizeof(AdjEdge));
	int u, v;
	double weight;
	for (int i = 0; i < M; i++) {
		fscanf(f, "%d %d %lf", &u, &v, &weight);
		adjEdges[i].weight = weight;
		adjEdges[i].v = v;
		adjEdges[i].next = adjNodes[u];
		adjNodes[u] = i;
		adjEdges[i + M].weight = weight;
		adjEdges[i + M].v = u;
		adjEdges[i + M].next = adjNodes[v];
		adjNodes[v] = i + M;
	}
	fclose(f);

	//convert adjacent list to CRS format
	nodes = (int *)malloc((N+1) * sizeof(int));
	edges = (Edge *)malloc(2 * M * sizeof(Edge));
	int edgeLen = 0;
	nodes[0] = 0;
	for(int i = 0; i < N; i++) {
		int edgeIdx = adjNodes[i];
		while(edgeIdx != -1) {
			AdjEdge adjEdge = adjEdges[edgeIdx];
			edges[edgeLen].v = adjEdge.v;
			edges[edgeLen].weight = adjEdge.weight;
			edgeLen++;
			edgeIdx = adjEdges[edgeIdx].next;
		}
		nodes[i+1] = edgeLen;
	}
	free(adjNodes);
	free(adjEdges);
}


int main(int argc, char *argv[]) { 
	clock_t start, end;
	start = clock();
    TCLAP::CmdLine cmd("Command description message", ' ', "0.1");    
    TCLAP::ValueArg<string> graphArg("g", "graph", "the path of the input graph", true, "", "string");
    TCLAP::ValueArg<string> nodeArg("n", "node_bc", "the path of node bc result", false, "", "string");
    TCLAP::ValueArg<string> edgeArg("e", "edge_bc", "the path of edge bc result", false, "", "string");
    TCLAP::ValueArg<int> warpArg("w", "warp_size", "the warp size if using warp-centric method", false, 32, "int");
    cmd.add(graphArg);
    cmd.add(nodeArg);
    cmd.add(edgeArg);
    cmd.add(warpArg);
    cmd.parse(argc, argv);
    string fpath = graphArg.getValue();
    string bcpath = nodeArg.getValue();
    string ebcpath = edgeArg.getValue();
    const int warp_size = warpArg.getValue();
    if(bcpath.empty() && ebcpath.empty()) {
        cout << "you must set at least one of node bc path and edge bc path" << endl;
        return 0;
    }
    bool calNode = false, calEdge = false;
    if(! bcpath.empty()) {
        calNode = true;
    }
    if(! ebcpath.empty()) {
        calEdge = true;
    }
    if(warp_size != 32 && warp_size != 16 && warp_size != 8 && warp_size != 4 && warp_size != 2 && warp_size != 1) {
        cout << "please choose a proper warp size in (2, 4, 8, 16, 32)" << endl;
        return 0;
    }
	buildGraph(fpath);
    BC(N, M, nodes, edges, calNode, calEdge, bcpath.c_str(), ebcpath.c_str(), warp_size);
	end = clock();
	cout << "total time: " << (double)(end - start) / CLOCKS_PER_SEC << "s" << endl;
	return 0;
}
